
/*
 * Copyright (C) 2010 Tudor Marian <tudor.marian@cornell.edu>
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <string.h>

#define LINESZ	1024
#define SYNC_BITS	2
#define PAYLOAD_BITS 64
#define BITS_PER_LINE (SYNC_BITS+PAYLOAD_BITS)

char buf[LINESZ];

static inline int parse_bit(char bit)
{
	switch (bit)
	{
		case '0':
			return 0;
		case '1':
			return 1;
		default:
			fprintf(stderr, "Bug: bit not 0 or 1\n");
			exit(EXIT_FAILURE);	
	}
	return -1;
}

int main(int argc, char *argv[])
{
	FILE *f;
	char *p;
	int i, j, sync, err = EXIT_SUCCESS;
	uint64_t payload;
	
	if (argc != 2)
	{
		fprintf(stderr, "Usage %s <file>\n", argv[0]);
		exit(EXIT_FAILURE);
	}
	
	if (!(f = fopen(argv[1], "r")))
	{
		fprintf(stderr, "Unable to open file %s\n", argv[1]);
		exit(EXIT_FAILURE);
	}
	
	fprintf(stdout, "hbitstream = [\\\n");
	while (fgets(buf, LINESZ, f))
	{
		int len = strlen(buf);
		
		/* skip over leading spaces */
		for (p = buf; *p != '\0' && (*p == ' ' || *p == '\t'); p++)
			;
		
		/* remove trailing '\n' */
		if (p[len-1] != '\n')
		{
			fprintf(stderr, "Bug, no new line at the end of file\n");
			err = EXIT_FAILURE;
			goto out;
		}
		p[--len] = '\0';
		
		/* remove trailing space */
		for (; p[len-1] == ' ' || p[len-1] == '\t'; len--)
			;
		
		/* ignore comments or empty strings */
		if (*p == '\0' || *p == '#')
			continue;
		
		if (len < BITS_PER_LINE)
		{
			fprintf(stderr, "Less than %d bits per line, skipping\n", BITS_PER_LINE);
			continue;
		}
		
		if (len > BITS_PER_LINE)
		{
			fprintf(stderr, "Bug, more than %d bits per line\n", BITS_PER_LINE);
			err = EXIT_FAILURE;
			goto out;
		}

		sync = parse_bit(*p++);
		sync = (sync << 1) | parse_bit(*p++);
		
		payload = 0;
		for (i = 0; i < PAYLOAD_BITS / 8; i++)
			for (j = (i*8+8)-1; j >= i*8; j--)
				payload = (payload << 1) | parse_bit(p[j]);
		fprintf(stdout,"(0x%1x, 0x%016llx),\\\n", sync, (long long) payload);
	}
	out:
	fprintf(stdout, "]\n");
	
	fclose(f);
	exit(err);
}
