
/*
 * Copyright (C) 2010 Tudor Marian <tudor.marian@cornell.edu>
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <string.h>

#include "common64b66b.h"

#define SYNC_BITS	2
#define PAYLOAD_BITS 64
#define BITS_PER_LINE (SYNC_BITS+PAYLOAD_BITS)
#define LINESZ (BITS_PER_LINE + 2) /* add new line and '\0' */

char buf[LINESZ];

static inline char parse_bit(int bit)
{
	switch (bit)
	{
		case 0:
			return '0';
		case 1:
			return '1';
		default:
			fprintf(stderr, "Bug: bit not 0 or 1\n");
			exit(EXIT_FAILURE);
	}
	return -1;
}

int main(int argc, char *argv[])
{
	int i, j, k, frames_cnt;
	struct synced_frame *frames;
	
	if (argc != 2)
	{
		fprintf(stderr, "Usage %s <bitstream file>\n", argv[0]);
		exit(EXIT_FAILURE);
	}
	
	if ((frames_cnt = bitstream_from_file(argv[1], &frames)) < 0)
	{
		fprintf(stderr, "Unable to load bitstream from file %s:%s\n", argv[1], 
			strerror(-frames_cnt));
		exit(EXIT_FAILURE);
	}
	
	for (k = 0; k < frames_cnt; k++)
	{
		char *p;
		/* issue sync first */
		buf[0] = parse_bit(frames[k].prefix >> 1);
		buf[1] = parse_bit(frames[k].prefix & 0x1);
		
		p = &buf[2];
		for (i = (PAYLOAD_BITS/8)-1; i >= 0; i--)
			for (j = i*8; j < (i*8+8); j++)
				*p++ = parse_bit((frames[k].payload >> j) & 0x1);
		*p = '\0';
		fprintf(stdout, "%s\n", buf);
	}
	
	free(frames);
	exit(EXIT_SUCCESS);
}
