
# Copyright (C) 2009 Tudor Marian <tudor.marian@cornell.edu>

import sys

code_group_name_idx = 0
octet_value_idx = 1
octet_bits_idx = 2
rd_minus_idx = 3
rd_plus_idx = 4

def error_propagation(name):
	return name == "K30.7"

def carrier_extended(name):
	return name == "K23.7"

def start_of_packet(name):
	return name == "K27.7"

def end_of_packet(name):
	return name == "K29.7"

def config_or_idle(name):
	return name == "K28.5"

def config_and_not_idle(name):
	return name == "D21.5" or name == "D2.2"

def idle1_or_2(name):
	return name == "D5.6" or name == "D16.2"

class CodeGroupsException(Exception):
	def __init__(self, value):
		self.value = value
	def __str__(self):
		return repr(self.value)
#end class DecodeException

def code_groups_map(data_code_groups, special_code_groups):
	map = {}
	for (name, octet_val, octet_bits, rd_minus, rd_plus) in data_code_groups:
		if rd_minus == rd_plus:
			if rd_minus in map:
				raise CodeGroupsException('rd_* 0x%x dup' % (rd_minus))
			map[rd_minus] = (name, octet_val, False)
		else:
			if rd_minus in map:
				raise CodeGroupsException('rd_minus 0x%x dup' % (rd_minus))
			map[rd_minus] = (name, octet_val, False)
			if rd_plus in map:
				raise CodeGroupsException('rd_plus 0x%x dup' % (rd_plus))
			map[rd_plus] = (name, octet_val, False)
	for (name, octet_val, octet_bits, rd_minus, rd_plus) in special_code_groups:
		if rd_minus == rd_plus:
			if rd_minus in map:
				raise CodeGroupsException('rd_*(Sp) 0x%x dup' % (rd_minus))
			map[rd_minus] = (name, octet_val, True)
		else:
			if rd_minus in map:
				raise CodeGroupsException('rd_minus(S) 0x%x dup' % (rd_minus))
			map[rd_minus] = (name, octet_val, True)
			if rd_plus in map:
				raise CodeGroupsException('rd_plus(S) 0x%x dup' % (rd_plus))
			map[rd_plus] = (name, octet_val, True)
	return map
# this script will actually generate the following two lists

data_code_groups=[\
("D0.0", 0x0, 0x0, 0x274, 0x18b),\
("D1.0", 0x1, 0x1, 0x1d4, 0x22b),\
("D2.0", 0x2, 0x2, 0x2d4, 0x12b),\
("D3.0", 0x3, 0x3, 0x31b, 0x314),\
("D4.0", 0x4, 0x4, 0x354, 0x0ab),\
("D5.0", 0x5, 0x5, 0x29b, 0x294),\
("D6.0", 0x6, 0x6, 0x19b, 0x194),\
("D7.0", 0x7, 0x7, 0x38b, 0x074),\
("D8.0", 0x8, 0x8, 0x394, 0x06b),\
("D9.0", 0x9, 0x9, 0x25b, 0x254),\
("D10.0", 0xa, 0xa, 0x15b, 0x154),\
("D11.0", 0xb, 0xb, 0x34b, 0x344),\
("D12.0", 0xc, 0xc, 0x0db, 0x0d4),\
("D13.0", 0xd, 0xd, 0x2cb, 0x2c4),\
("D14.0", 0xe, 0xe, 0x1cb, 0x1c4),\
("D15.0", 0xf, 0xf, 0x174, 0x28b),\
("D16.0", 0x10, 0x10, 0x1b4, 0x24b),\
("D17.0", 0x11, 0x11, 0x23b, 0x234),\
("D18.0", 0x12, 0x12, 0x13b, 0x134),\
("D19.0", 0x13, 0x13, 0x32b, 0x324),\
("D20.0", 0x14, 0x14, 0x0bb, 0x0b4),\
("D21.0", 0x15, 0x15, 0x2ab, 0x2a4),\
("D22.0", 0x16, 0x16, 0x1ab, 0x1a4),\
("D23.0", 0x17, 0x17, 0x3a4, 0x05b),\
("D24.0", 0x18, 0x18, 0x334, 0x0cb),\
("D25.0", 0x19, 0x19, 0x26b, 0x264),\
("D26.0", 0x1a, 0x1a, 0x16b, 0x164),\
("D27.0", 0x1b, 0x1b, 0x364, 0x09b),\
("D28.0", 0x1c, 0x1c, 0x0eb, 0x0e4),\
("D29.0", 0x1d, 0x1d, 0x2e4, 0x11b),\
("D30.0", 0x1e, 0x1e, 0x1e4, 0x21b),\
("D31.0", 0x1f, 0x1f, 0x2b4, 0x14b),\
("D0.1", 0x20, 0x20, 0x279, 0x189),\
("D1.1", 0x21, 0x21, 0x1d9, 0x229),\
("D2.1", 0x22, 0x22, 0x2d9, 0x129),\
("D3.1", 0x23, 0x23, 0x319, 0x319),\
("D4.1", 0x24, 0x24, 0x359, 0x0a9),\
("D5.1", 0x25, 0x25, 0x299, 0x299),\
("D6.1", 0x26, 0x26, 0x199, 0x199),\
("D7.1", 0x27, 0x27, 0x389, 0x079),\
("D8.1", 0x28, 0x28, 0x399, 0x069),\
("D9.1", 0x29, 0x29, 0x259, 0x259),\
("D10.1", 0x2a, 0x2a, 0x159, 0x159),\
("D11.1", 0x2b, 0x2b, 0x349, 0x349),\
("D12.1", 0x2c, 0x2c, 0x0d9, 0x0d9),\
("D13.1", 0x2d, 0x2d, 0x2c9, 0x2c9),\
("D14.1", 0x2e, 0x2e, 0x1c9, 0x1c9),\
("D15.1", 0x2f, 0x2f, 0x179, 0x289),\
("D16.1", 0x30, 0x30, 0x1b9, 0x249),\
("D17.1", 0x31, 0x31, 0x239, 0x239),\
("D18.1", 0x32, 0x32, 0x139, 0x139),\
("D19.1", 0x33, 0x33, 0x329, 0x329),\
("D20.1", 0x34, 0x34, 0x0b9, 0x0b9),\
("D21.1", 0x35, 0x35, 0x2a9, 0x2a9),\
("D22.1", 0x36, 0x36, 0x1a9, 0x1a9),\
("D23.1", 0x37, 0x37, 0x3a9, 0x059),\
("D24.1", 0x38, 0x38, 0x339, 0x0c9),\
("D25.1", 0x39, 0x39, 0x269, 0x269),\
("D26.1", 0x3a, 0x3a, 0x169, 0x169),\
("D27.1", 0x3b, 0x3b, 0x369, 0x099),\
("D28.1", 0x3c, 0x3c, 0x0e9, 0x0e9),\
("D29.1", 0x3d, 0x3d, 0x2e9, 0x119),\
("D30.1", 0x3e, 0x3e, 0x1e9, 0x219),\
("D31.1", 0x3f, 0x3f, 0x2b9, 0x149),\
("D0.2", 0x40, 0x40, 0x275, 0x185),\
("D1.2", 0x41, 0x41, 0x1d5, 0x225),\
("D2.2", 0x42, 0x42, 0x2d5, 0x125),\
("D3.2", 0x43, 0x43, 0x315, 0x315),\
("D4.2", 0x44, 0x44, 0x355, 0x0a5),\
("D5.2", 0x45, 0x45, 0x295, 0x295),\
("D6.2", 0x46, 0x46, 0x195, 0x195),\
("D7.2", 0x47, 0x47, 0x385, 0x075),\
("D8.2", 0x48, 0x48, 0x395, 0x065),\
("D9.2", 0x49, 0x49, 0x255, 0x255),\
("D10.2", 0x4a, 0x4a, 0x155, 0x155),\
("D11.2", 0x4b, 0x4b, 0x345, 0x345),\
("D12.2", 0x4c, 0x4c, 0x0d5, 0x0d5),\
("D13.2", 0x4d, 0x4d, 0x2c5, 0x2c5),\
("D14.2", 0x4e, 0x4e, 0x1c5, 0x1c5),\
("D15.2", 0x4f, 0x4f, 0x175, 0x285),\
("D16.2", 0x50, 0x50, 0x1b5, 0x245),\
("D17.2", 0x51, 0x51, 0x235, 0x235),\
("D18.2", 0x52, 0x52, 0x135, 0x135),\
("D19.2", 0x53, 0x53, 0x325, 0x325),\
("D20.2", 0x54, 0x54, 0x0b5, 0x0b5),\
("D21.2", 0x55, 0x55, 0x2a5, 0x2a5),\
("D22.2", 0x56, 0x56, 0x1a5, 0x1a5),\
("D23.2", 0x57, 0x57, 0x3a5, 0x055),\
("D24.2", 0x58, 0x58, 0x335, 0x0c5),\
("D25.2", 0x59, 0x59, 0x265, 0x265),\
("D26.2", 0x5a, 0x5a, 0x165, 0x165),\
("D27.2", 0x5b, 0x5b, 0x365, 0x095),\
("D28.2", 0x5c, 0x5c, 0x0e5, 0x0e5),\
("D29.2", 0x5d, 0x5d, 0x2e5, 0x115),\
("D30.2", 0x5e, 0x5e, 0x1e5, 0x215),\
("D31.2", 0x5f, 0x5f, 0x2b5, 0x145),\
("D0.3", 0x60, 0x60, 0x273, 0x18c),\
("D1.3", 0x61, 0x61, 0x1d3, 0x22c),\
("D2.3", 0x62, 0x62, 0x2d3, 0x12c),\
("D3.3", 0x63, 0x63, 0x31c, 0x313),\
("D4.3", 0x64, 0x64, 0x353, 0x0ac),\
("D5.3", 0x65, 0x65, 0x29c, 0x293),\
("D6.3", 0x66, 0x66, 0x19c, 0x193),\
("D7.3", 0x67, 0x67, 0x38c, 0x073),\
("D8.3", 0x68, 0x68, 0x393, 0x06c),\
("D9.3", 0x69, 0x69, 0x25c, 0x253),\
("D10.3", 0x6a, 0x6a, 0x15c, 0x153),\
("D11.3", 0x6b, 0x6b, 0x34c, 0x343),\
("D12.3", 0x6c, 0x6c, 0x0dc, 0x0d3),\
("D13.3", 0x6d, 0x6d, 0x2cc, 0x2c3),\
("D14.3", 0x6e, 0x6e, 0x1cc, 0x1c3),\
("D15.3", 0x6f, 0x6f, 0x173, 0x28c),\
("D16.3", 0x70, 0x70, 0x1b3, 0x24c),\
("D17.3", 0x71, 0x71, 0x23c, 0x233),\
("D18.3", 0x72, 0x72, 0x13c, 0x133),\
("D19.3", 0x73, 0x73, 0x32c, 0x323),\
("D20.3", 0x74, 0x74, 0x0bc, 0x0b3),\
("D21.3", 0x75, 0x75, 0x2ac, 0x2a3),\
("D22.3", 0x76, 0x76, 0x1ac, 0x1a3),\
("D23.3", 0x77, 0x77, 0x3a3, 0x05c),\
("D24.3", 0x78, 0x78, 0x333, 0x0cc),\
("D25.3", 0x79, 0x79, 0x26c, 0x263),\
("D26.3", 0x7a, 0x7a, 0x16c, 0x163),\
("D27.3", 0x7b, 0x7b, 0x363, 0x09c),\
("D28.3", 0x7c, 0x7c, 0x0ec, 0x0e3),\
("D29.3", 0x7d, 0x7d, 0x2e3, 0x11c),\
("D30.3", 0x7e, 0x7e, 0x1e3, 0x21c),\
("D31.3", 0x7f, 0x7f, 0x2b3, 0x14c),\
("D0.4", 0x80, 0x80, 0x272, 0x18d),\
("D1.4", 0x81, 0x81, 0x1d2, 0x22d),\
("D2.4", 0x82, 0x82, 0x2d2, 0x12d),\
("D3.4", 0x83, 0x83, 0x31d, 0x312),\
("D4.4", 0x84, 0x84, 0x352, 0x0ad),\
("D5.4", 0x85, 0x85, 0x29d, 0x292),\
("D6.4", 0x86, 0x86, 0x19d, 0x192),\
("D7.4", 0x87, 0x87, 0x38d, 0x072),\
("D8.4", 0x88, 0x88, 0x392, 0x06d),\
("D9.4", 0x89, 0x89, 0x25d, 0x252),\
("D10.4", 0x8a, 0x8a, 0x15d, 0x152),\
("D11.4", 0x8b, 0x8b, 0x34d, 0x342),\
("D12.4", 0x8c, 0x8c, 0x0dd, 0x0d2),\
("D13.4", 0x8d, 0x8d, 0x2cd, 0x2c2),\
("D14.4", 0x8e, 0x8e, 0x1cd, 0x1c2),\
("D15.4", 0x8f, 0x8f, 0x172, 0x28d),\
("D16.4", 0x90, 0x90, 0x1b2, 0x24d),\
("D17.4", 0x91, 0x91, 0x23d, 0x232),\
("D18.4", 0x92, 0x92, 0x13d, 0x132),\
("D19.4", 0x93, 0x93, 0x32d, 0x322),\
("D20.4", 0x94, 0x94, 0x0bd, 0x0b2),\
("D21.4", 0x95, 0x95, 0x2ad, 0x2a2),\
("D22.4", 0x96, 0x96, 0x1ad, 0x1a2),\
("D23.4", 0x97, 0x97, 0x3a2, 0x05d),\
("D24.4", 0x98, 0x98, 0x332, 0x0cd),\
("D25.4", 0x99, 0x99, 0x26d, 0x262),\
("D26.4", 0x9a, 0x9a, 0x16d, 0x162),\
("D27.4", 0x9b, 0x9b, 0x362, 0x09d),\
("D28.4", 0x9c, 0x9c, 0x0ed, 0x0e2),\
("D29.4", 0x9d, 0x9d, 0x2e2, 0x11d),\
("D30.4", 0x9e, 0x9e, 0x1e2, 0x21d),\
("D31.4", 0x9f, 0x9f, 0x2b2, 0x14d),\
("D0.5", 0xa0, 0xa0, 0x27a, 0x18a),\
("D1.5", 0xa1, 0xa1, 0x1da, 0x22a),\
("D2.5", 0xa2, 0xa2, 0x2da, 0x12a),\
("D3.5", 0xa3, 0xa3, 0x31a, 0x31a),\
("D4.5", 0xa4, 0xa4, 0x35a, 0x0aa),\
("D5.5", 0xa5, 0xa5, 0x29a, 0x29a),\
("D6.5", 0xa6, 0xa6, 0x19a, 0x19a),\
("D7.5", 0xa7, 0xa7, 0x38a, 0x07a),\
("D8.5", 0xa8, 0xa8, 0x39a, 0x06a),\
("D9.5", 0xa9, 0xa9, 0x25a, 0x25a),\
("D10.5", 0xaa, 0xaa, 0x15a, 0x15a),\
("D11.5", 0xab, 0xab, 0x34a, 0x34a),\
("D12.5", 0xac, 0xac, 0x0da, 0x0da),\
("D13.5", 0xad, 0xad, 0x2ca, 0x2ca),\
("D14.5", 0xae, 0xae, 0x1ca, 0x1ca),\
("D15.5", 0xaf, 0xaf, 0x17a, 0x28a),\
("D16.5", 0xb0, 0xb0, 0x1ba, 0x24a),\
("D17.5", 0xb1, 0xb1, 0x23a, 0x23a),\
("D18.5", 0xb2, 0xb2, 0x13a, 0x13a),\
("D19.5", 0xb3, 0xb3, 0x32a, 0x32a),\
("D20.5", 0xb4, 0xb4, 0x0ba, 0x0ba),\
("D21.5", 0xb5, 0xb5, 0x2aa, 0x2aa),\
("D22.5", 0xb6, 0xb6, 0x1aa, 0x1aa),\
("D23.5", 0xb7, 0xb7, 0x3aa, 0x05a),\
("D24.5", 0xb8, 0xb8, 0x33a, 0x0ca),\
("D25.5", 0xb9, 0xb9, 0x26a, 0x26a),\
("D26.5", 0xba, 0xba, 0x16a, 0x16a),\
("D27.5", 0xbb, 0xbb, 0x36a, 0x09a),\
("D28.5", 0xbc, 0xbc, 0x0ea, 0x0ea),\
("D29.5", 0xbd, 0xbd, 0x2ea, 0x11a),\
("D30.5", 0xbe, 0xbe, 0x1ea, 0x21a),\
("D31.5", 0xbf, 0xbf, 0x2ba, 0x14a),\
("D0.6", 0xc0, 0xc0, 0x276, 0x186),\
("D1.6", 0xc1, 0xc1, 0x1d6, 0x226),\
("D2.6", 0xc2, 0xc2, 0x2d6, 0x126),\
("D3.6", 0xc3, 0xc3, 0x316, 0x316),\
("D4.6", 0xc4, 0xc4, 0x356, 0x0a6),\
("D5.6", 0xc5, 0xc5, 0x296, 0x296),\
("D6.6", 0xc6, 0xc6, 0x196, 0x196),\
("D7.6", 0xc7, 0xc7, 0x386, 0x076),\
("D8.6", 0xc8, 0xc8, 0x396, 0x066),\
("D9.6", 0xc9, 0xc9, 0x256, 0x256),\
("D10.6", 0xca, 0xca, 0x156, 0x156),\
("D11.6", 0xcb, 0xcb, 0x346, 0x346),\
("D12.6", 0xcc, 0xcc, 0x0d6, 0x0d6),\
("D13.6", 0xcd, 0xcd, 0x2c6, 0x2c6),\
("D14.6", 0xce, 0xce, 0x1c6, 0x1c6),\
("D15.6", 0xcf, 0xcf, 0x176, 0x286),\
("D16.6", 0xd0, 0xd0, 0x1b6, 0x246),\
("D17.6", 0xd1, 0xd1, 0x236, 0x236),\
("D18.6", 0xd2, 0xd2, 0x136, 0x136),\
("D19.6", 0xd3, 0xd3, 0x326, 0x326),\
("D20.6", 0xd4, 0xd4, 0x0b6, 0x0b6),\
("D21.6", 0xd5, 0xd5, 0x2a6, 0x2a6),\
("D22.6", 0xd6, 0xd6, 0x1a6, 0x1a6),\
("D23.6", 0xd7, 0xd7, 0x3a6, 0x056),\
("D24.6", 0xd8, 0xd8, 0x336, 0x0c6),\
("D25.6", 0xd9, 0xd9, 0x266, 0x266),\
("D26.6", 0xda, 0xda, 0x166, 0x166),\
("D27.6", 0xdb, 0xdb, 0x366, 0x096),\
("D28.6", 0xdc, 0xdc, 0x0e6, 0x0e6),\
("D29.6", 0xdd, 0xdd, 0x2e6, 0x116),\
("D30.6", 0xde, 0xde, 0x1e6, 0x216),\
("D31.6", 0xdf, 0xdf, 0x2b6, 0x146),\
("D0.7", 0xe0, 0xe0, 0x271, 0x18e),\
("D1.7", 0xe1, 0xe1, 0x1d1, 0x22e),\
("D2.7", 0xe2, 0xe2, 0x2d1, 0x12e),\
("D3.7", 0xe3, 0xe3, 0x31e, 0x311),\
("D4.7", 0xe4, 0xe4, 0x351, 0x0ae),\
("D5.7", 0xe5, 0xe5, 0x29e, 0x291),\
("D6.7", 0xe6, 0xe6, 0x19e, 0x191),\
("D7.7", 0xe7, 0xe7, 0x38e, 0x071),\
("D8.7", 0xe8, 0xe8, 0x391, 0x06e),\
("D9.7", 0xe9, 0xe9, 0x25e, 0x251),\
("D10.7", 0xea, 0xea, 0x15e, 0x151),\
("D11.7", 0xeb, 0xeb, 0x34e, 0x348),\
("D12.7", 0xec, 0xec, 0x0de, 0x0d1),\
("D13.7", 0xed, 0xed, 0x2ce, 0x2c8),\
("D14.7", 0xee, 0xee, 0x1ce, 0x1c8),\
("D15.7", 0xef, 0xef, 0x171, 0x28e),\
("D16.7", 0xf0, 0xf0, 0x1b1, 0x24e),\
("D17.7", 0xf1, 0xf1, 0x237, 0x231),\
("D18.7", 0xf2, 0xf2, 0x137, 0x131),\
("D19.7", 0xf3, 0xf3, 0x32e, 0x321),\
("D20.7", 0xf4, 0xf4, 0x0b7, 0x0b1),\
("D21.7", 0xf5, 0xf5, 0x2ae, 0x2a1),\
("D22.7", 0xf6, 0xf6, 0x1ae, 0x1a1),\
("D23.7", 0xf7, 0xf7, 0x3a1, 0x05e),\
("D24.7", 0xf8, 0xf8, 0x331, 0x0ce),\
("D25.7", 0xf9, 0xf9, 0x26e, 0x261),\
("D26.7", 0xfa, 0xfa, 0x16e, 0x161),\
("D27.7", 0xfb, 0xfb, 0x361, 0x09e),\
("D28.7", 0xfc, 0xfc, 0x0ee, 0x0e1),\
("D29.7", 0xfd, 0xfd, 0x2e1, 0x11e),\
("D30.7", 0xfe, 0xfe, 0x1e1, 0x21e),\
("D31.7", 0xff, 0xff, 0x2b1, 0x14e),\
]
special_code_groups=[\
("K28.0", 0x1c, 0x1c, 0x0f4, 0x30b),\
("K28.1", 0x3c, 0x3c, 0x0f9, 0x306),\
("K28.2", 0x5c, 0x5c, 0x0f5, 0x30a),\
("K28.3", 0x7c, 0x7c, 0x0f3, 0x30c),\
("K28.4", 0x9c, 0x9c, 0x0f2, 0x30d),\
("K28.5", 0xbc, 0xbc, 0x0fa, 0x305),\
("K28.6", 0xdc, 0xdc, 0x0f6, 0x309),\
("K28.7", 0xfc, 0xfc, 0x0f8, 0x307),\
("K23.7", 0xf7, 0xf7, 0x3a8, 0x057),\
("K27.7", 0xfb, 0xfb, 0x368, 0x097),\
("K29.7", 0xfd, 0xfd, 0x2e8, 0x117),\
("K30.7", 0xfe, 0xfe, 0x1e8, 0x217),\
]

def main():
	if len(sys.argv) != 3:
		sys.stderr.write('Usage %s <data-code-groups> <special-code-groups>\n' \
			% (sys.argv[0]))
		sys.exit(1)
	
	f=open(sys.argv[1], 'r')
	sys.stdout.write('data_code_groups=[\\\n')
	for line in f:
		if not line:
			continue
		pieces = line.split()
		s='("%s", 0x%x, 0x%x, 0x%03x, 0x%03x)' % \
			(pieces[code_group_name_idx], int(pieces[octet_value_idx], 16), \
			int(pieces[octet_bits_idx], 2), int(pieces[rd_minus_idx], 2), \
			int(pieces[rd_plus_idx], 2))
		sys.stdout.write('%s,\\\n' % (s))
	sys.stdout.write(']\n')
	f.close()
	
	f=open(sys.argv[2], 'r')
	sys.stdout.write('special_code_groups=[\\\n')
	for line in f:
		if not line:
			continue
		pieces = line.split()
		s='("%s", 0x%x, 0x%x, 0x%03x, 0x%03x)' % \
			(pieces[code_group_name_idx], int(pieces[octet_value_idx], 16), \
			int(pieces[octet_bits_idx], 2), int(pieces[rd_minus_idx], 2), \
			int(pieces[rd_plus_idx], 2))
		sys.stdout.write('%s,\\\n' % (s))
	sys.stdout.write(']\n')
	f.close()
#end main

if __name__ == "__main__":
	main()
