
# Copyright (C) 2009 Tudor Marian <tudor.marian@cornell.edu>

import os,sys
from decodecommon import *
from scramblercommon import *
from common64b66b import *

def print_control_verbose(str):
	sys.stderr.write(str)
#end print_control_verbose
def print_control_void(str):
	pass
#end print_control_void
print_control = print_control_void

def parse_control_codes(range_from, range_to, codeword):
	error = False
	reserved = False
	control_codes = 'control_codes: '
	i = 7-range_from # for i in range(7-range_from, 7-1-range_to, -1):
	while i > (7-1-range_to):
		control_code = (codeword >> (i * 7)) & 0x7f
		control_codes += '0x%x ' % control_code
		if control_code == 0x00:
			pass
		elif control_code == 0x1e:
			error = True
		else:
			error = True
			reserved = True
		i -= 1
	control_codes += '\n'
	print_control(control_codes)
	return (error, reserved)
#end parse_control_codes

def parse_bytes(range_from, range_to, codeword):
	bytes = ''
	i = 7-range_from # for i in range(7-range_from, 7-1-range_to, -1):
	while i > (7-1-range_to):
		byte = (codeword >> (i * 8)) & 0xff
		bytes += chr(byte)
		i -= 1
	return bytes
#end parse_bytes 

# python does not have tail recursion optimization --- do it by hand
def do_decode(state, descrambler_state, vector, param, accept_fn):
	if not vector:
		return None
	
	frame_bytes, bit_position, frame_start, frame_end, idles = param
	sync_prefix = vector[0][0]
	scrambled_codeword = vector[0][1]
	
	if (sync_prefix != 0x2) and (sync_prefix != 0x1):
		sys.stderr.write('Invalid sync preamble, skipping\n')
		# keep old descrambler state
		param = (frame_bytes, bit_position+66, None, None, idles)
		del vector[0] # peell off mutable list instead of passing a copy vector[1:]
		return ('start', descrambler_state, vector, param)
	
	if state not in states:
		raise DecodeException('Invalid state, terminating\n')

	if state is 'error': # the error message is in param
		sys.stderr.write(str(frame_bytes))
		param = ('', bit_position+66, None, None, idles)
		del vector[0]
		return ('start', descrambler_state, vector, param)
	elif state is 'start':
		if sync_prefix == 0x1:
			err = 'Started with data codeword\n'
			param = (err, bit_position, frame_start, frame_end, idles)
			return ('error', descrambler_state, vector, param)
		else:
			new_descrambler_state, codeword = \
				scrambler(descrambler_state, scrambled_codeword, descramble)
			type = (codeword >> (64-8)) & 0xff
			if type == 0x1e:
				error, reserved = parse_control_codes(0, 7, codeword)
				if error or reserved:
					err = 'Error %s, reserved %s\n' % (repr(error), repr(reserved))
					param = (err, bit_position, frame_start, frame_end, idles)
					return ('error', new_descrambler_state, vector, param)
				else:
					print_control('zzzz (%d idle characters)\n' % 8)
					param = ('', bit_position+66, frame_start, frame_end, idles+8)
					del vector[0]
					return ('start', new_descrambler_state, vector, param)
			elif type == 0x33:
				error, reserved = parse_control_codes(0, 3, codeword)
				if error or reserved:
					err = 'Error %s, reserved %s\n' % (repr(error), repr(reserved))
					param = (err, bit_position, frame_start, frame_end, idles)
					return ('error', new_descrambler_state, vector, param)
				else:
					print_control('zzzz (%d idle characters)\n' % 4)
					bytes = parse_bytes(5, 7, codeword)
					param = (frame_bytes+bytes, bit_position+66, \
						bit_position+(66-3*8), None, idles+(8-3))
					del vector[0]
					return ('consume-frame', new_descrambler_state, vector, param)
			elif type == 0x78:
				bytes = parse_bytes(1, 7, codeword)
				param = (frame_bytes+bytes, bit_position+66, bit_position+2+8, \
					None, idles+1)
				del vector[0]
				return ('consume-frame', new_descrambler_state, vector, param)
			elif type in [0x66, 0x55, 0x4b, 0x2d]:
				err = 'Ordered sets error\n'
				param = (err, bit_position, frame_start, frame_end, idles)
				return ('error', new_descrambler_state, vector, param)
			else:
				err = 'Invalid type to begin with (probably out of sync)\n'
				param = (err, bit_position, frame_start, frame_end, idles)
				return ('error', new_descrambler_state, vector, param)
	elif state is 'consume-frame':
		new_descrambler_state, codeword = \
				scrambler(descrambler_state, scrambled_codeword, descramble)
		if sync_prefix == 0x1:
			bytes = parse_bytes(0, 7, codeword)
			param = (frame_bytes+bytes, bit_position+66, frame_start, frame_end, idles)
			del vector[0]
			return ('consume-frame', new_descrambler_state, vector, param)
		else:
			type = (codeword >> (64-8)) & 0xff
			if type == 0x87:
				control_codes = 7
			elif type == 0x99:
				control_codes = 6
			elif type == 0xaa:
				control_codes = 5
			elif type == 0xb4:
				control_codes = 4
			elif type == 0xcc:
				control_codes = 3
			elif type == 0xd2:
				control_codes = 2
			elif type == 0xe1:
				control_codes = 1
			elif type == 0xff:
				control_codes = 0
			else:
				err = 'Invalid type in consume-frame state %x' % type
				param = (err, bit_position, frame_start, frame_end, idles)
				return ('error', new_descrambler_state, vector, param)
				
			error, reserved = parse_control_codes(8-control_codes, 7, codeword)
			if error or reserved:
				err = 'Error %s, reserved %s\n' % (repr(error), repr(reserved))
				param = (err, bit_position, frame_start, frame_end, idles)
				return ('error', new_descrambler_state, vector, param)
			else:
				print_control('zzzz (%d idle characters)\n' % control_codes)
				bytes = parse_bytes(1, 7-control_codes, codeword)
				param = (frame_bytes+bytes, bit_position, frame_start, \
					bit_position+2+8+(7-control_codes)*8, control_codes+1)
				return ('accept-frame', new_descrambler_state, vector, param)
	elif state is 'accept-frame':
		accept_ret = accept_fn(frame_bytes)
		if accept_ret:
			assert isinstance(accept_ret, tuple)
			if len(accept_ret) == 3:
				msg, iperf_id, ip_id = accept_ret
			else:
				assert len(accept_ret) == 4
				msg, ip_id, udp_payload, tabs = accept_ret
				iperf_id = -1
			sys.stdout.write('%s' % msg)
			sys.stdout.write('Iperf id %d , IP id %d , start bit %s , end bit %s\n'\
				% (iperf_id, ip_id, repr(frame_start), repr(frame_end)))
		param = ('', bit_position+66, None, None, idles)
		del vector[0]
		return ('start', descrambler_state, vector, param)
	else:
		raise DecodeException('Invalid state, terminating\n')

def decode_(state, descrambler_state, vector, accept_fn):
	param = ('', 0, None, None, 0)
	idle_gaps = []
	if not vector:
		sys.stderr.write('No input.\n')
	else:
		while True:
			ret = do_decode(state, descrambler_state, vector, param, accept_fn)
			if not ret:
				idle_gaps.append(param[4])
				return idle_gaps
			if state == 'start' and ret[0] == 'consume-frame':
				idle_gaps.append(ret[3][4])
			state, descrambler_state, vector, param = ret
#end decode_

def decode(vector, accept_fn):
	return decode_('start', 0xffffffffffffffff, vector, accept_fn)
#end decode

############# MAIN decoder functionality ends here ###########################

import cProfile,pstats

prof_params = { 'accept_fn' : print_iperf_vector_decoded }
packet_vector = []

def append_packets(packet):
	packet_vector.append(packet)
#end append_packets

def usage_exit():
	sys.stderr.write('Usage %s [-p (enable profiling)] '\
		'[-o (output pickled packet vector)] '\
		'[-n (do not attempt to parse iperf header)] ' \
		'<path to hbitstream module>\n' % sys.argv[0])
	sys.exit(1)
#end usage_exit

def run_decoder():
	bitstream = prof_params['bitstream']
	accept_fn = prof_params['accept_fn']
	if bitstream and accept_fn:
		idle_gaps = decode(bitstream, accept_fn)
		sys.stderr.write('Idle gaps: %s\n' % repr(idle_gaps))
		if accept_fn is append_packets:
			packet_list_to_file(packet_vector, sys.stdout)
	else:
		sys.stderr.write('Invalid parameters through profiling wrapper\n')
		sys.exit(1)
#end run_decoder

def main():
	try:
		if len(sys.argv) <= 1:
			usage_exit()
		i = 1
		noiperf = False
		do_profile = False
		path_to_hbitstream = None
		while i < len(sys.argv):
			if sys.argv[i] == "-p":
				do_profile = True
			elif sys.argv[i] == "-n":
				noiperf = True
			elif sys.argv[i] == "-o":
				prof_params['accept_fn'] = append_packets
			else:
				try:
					path_to_hbitstream = sys.argv[i]
					os.stat(path_to_hbitstream)
				except OSError, e:
					sys.stderr.write('Invalid path %s\n' % path_to_hbitstream)
					usage_exit()
			i += 1
		
		if noiperf:
			if prof_params['accept_fn'] == append_packets:
				sys.stderr.write('-o flag supersedes and cancels the -n flag\n')
			else:
				prof_params['accept_fn'] = print_eth_vector_decoded
		if not path_to_hbitstream:
			usage_exit()
		prof_params['bitstream'] = hbitstream_from_file(path_to_hbitstream)
		if do_profile:
			cProfile.run('run_decoder()', 'prof')
			p = pstats.Stats('prof')
			p.strip_dirs().sort_stats('time').print_stats(15)
		else:
			run_decoder()
	except DecodeException, e:
		sys.stderr.write(e)
#end main

if __name__ == "__main__":
	main()
