/*
 * Copyright (C) 2009 Tudor Marian <tudor.marian@cornell.edu>
 */

#include "common64b66b.h"

#ifndef _DECODECOMMON_H_
#define _DECODECOMMON_H_

struct iperf_id_and_ip_id_ret
{
	int iperf_id;
	uint16_t ip_id;
};

enum decode_states { error, start, consume_frame, accept_frame };

typedef void (*accept_fn)(struct frame_bytes *frame, void *param);

/* may use sprintf here since this function is called w/ MAX_MTU size buffer */
void raw_eth_vector_decoded(struct frame_bytes *frame, void *param)
{
	int i;
	char *buf = param;
	
	for (i = 0; i < frame->len; i++)
	{
		if (i == frame->len-1)
			sprintf(buf, "%02x\n", (unsigned char) frame->buf[i]);
		else
			sprintf(buf, "%02x,", (unsigned char) frame->buf[i]);
		buf += strlen(buf);
	}
}

#define MAX_TABS 16
/* param holds a sufficiently large buffer */
void print_eth_vector_decoded(struct frame_bytes *frame, void *param)
{
	struct iperf_id_and_ip_id_ret *ids = param;
	char *pad = param + sizeof(struct iperf_id_and_ip_id_ret), tab_pad[MAX_TABS];
	int i, tabs = 0;
	size_t len = frame->len;
	unsigned char *bytes = (unsigned char *)frame->buf, 
		*crc_bytes = (unsigned char *) &frame->buf[len-4];
	struct ether_header *ethh;
	struct ip *iph;
	struct udphdr *udph;
	uint32_t crc;
	uint16_t check;
	
	memset(tab_pad, 0, MAX_TABS);
	pad[0] = '\0';
	
	/* 6 x 0x55 bytes */
	for (i = 0; i < 6; i++)
		if (bytes[i] != 0x55)
		{
			fprintf(stderr, "Invalid preamble bytes\n");
			return;
		}
	/* SOF, little endian 10101011 */
	if (bytes[6] != 0xd5)
	{
		fprintf(stderr, "Invalid SOF\n");
		return;
	}
	ethh = (struct ether_header *) &bytes[7];
	
	if (ntohs(ethh->ether_type) != ETHER_TYPE_IPV4)
	{
		fprintf(stderr, "Invalid ETHER_TYPE: %d\n", ntohs(ethh->ether_type));
		return;
	}
	
	sprintf(pad, "%sMAC:\n", tab_pad);
	pad = &pad[strlen(pad)];
	tab_pad[tabs++] = '\t';
	
	sprintf(pad, "%sDst addr: "MACHEX_FMT", Src addr: "\
			MACHEX_FMT"\n\tEtherType: 0x%X\n", 
			tab_pad, MACHEX(ethh->ether_dhost), MACHEX(ethh->ether_shost), 
			ntohs(ethh->ether_type));
	pad = &pad[strlen(pad)];
	
	crc = *(uint32_t *) crc_bytes;
	if (crc == (crc32_le(INIT_CRC, (void *) ethh, 
				((unsigned long)crc_bytes) - ((unsigned long) ethh)) ^ INIT_CRC))
		sprintf(pad, "%sCRC: 0x%08x (correct)\n", tab_pad, crc);
	else
		sprintf(pad, "%sCRC: 0x%08x (INCORRECT!)\n", tab_pad, crc);
	pad = &pad[strlen(pad)];
	
	/* IP */
	iph = (void *) ethh + sizeof(struct ether_header);
	
	if (iph->ip_p != IPPROTO_UDP)
	{
		fprintf(stderr, "Only UDP payload decoding supported\n");
		return;
	}
	
	sprintf(pad, "\n%sIP:\n", tab_pad);
	pad = &pad[strlen(pad)];
	tab_pad[tabs++] = '\t';
	
	sprintf(pad, "%sVersion: %d, Hdr len %d, IP TOS: %d, IP total length: %d\n", 
			tab_pad, iph->ip_v, iph->ip_hl*4, iph->ip_tos, ntohs(iph->ip_len));
	pad = &pad[strlen(pad)];
	sprintf(pad, "%sID: %d, Flags: %x, Fragment offset: %d, TTL: %d\n", 
			tab_pad, ntohs(iph->ip_id), ntohs(iph->ip_off) >> (5+8), 
			ntohs(iph->ip_off) & 0x1fff, 
			iph->ip_ttl);
	pad = &pad[strlen(pad)];
	
	check = ntohs(iph->ip_sum);
	iph->ip_sum = 0;
	if (check == csum(iph, iph->ip_hl*4))
		sprintf(pad, "%sProtocol: %d, Hdr checksum: 0x%04x (correct)\n", 
			tab_pad, iph->ip_p, check);
	else
		sprintf(pad, "%sProtocol: %d, Hdr checksum: 0x%04x (INCORRECT: 0x%04x)\n", 
			tab_pad, iph->ip_p, check, csum(iph, iph->ip_hl*4));
	pad = &pad[strlen(pad)];

	sprintf(pad, "%sSrc addr: "NIPQUAD_FMT", Dst addr: "NIPQUAD_FMT"\n", 
		tab_pad, NIPQUAD(iph->ip_src), NIPQUAD(iph->ip_dst));
	pad = &pad[strlen(pad)];
	if ((iph->ip_hl*4) > sizeof(struct ip))
	{
		sprintf(pad, "%sOptions: %d bytes worth\n", tab_pad, 
				(int) (iph->ip_hl*4 -  sizeof(struct ip)));
		pad = &pad[strlen(pad)];
	}
	
	/* UDP */
	udph = (void *) iph + iph->ip_hl*4;
	sprintf(pad, "\n%sUDP:\n", tab_pad);
	pad = &pad[strlen(pad)];
	tab_pad[tabs++] = '\t';
	sprintf(pad, "%sSrc port: %d, Dst port: %d, Length: %d\n", 
		tab_pad, ntohs(udph->uh_sport), ntohs(udph->uh_dport), 
		ntohs(udph->uh_ulen));
	pad = &pad[strlen(pad)];
	
	check = ntohs(udph->uh_sum);
	udph->uh_sum = 0;
	if (check == udp_csum(udph, iph))
		sprintf(pad, "%sChecksum: 0x%04x (correct)\n", tab_pad, check);
	else
		sprintf(pad, "%sChecksum: 0x%04x (INCORRECT: 0x%04x)\n", tab_pad, check, 
				udp_csum(udph, iph));
	pad = &pad[strlen(pad)];
	
	/* prep the return value */
	ids->iperf_id = -1;
	ids->ip_id = ntohs(iph->ip_id);
}

void print_iperf_vector_decoded(struct frame_bytes *frame, void *param)
{
	struct iperf_id_and_ip_id_ret *ids = param;
	char *pad = param + sizeof(struct iperf_id_and_ip_id_ret), tab_pad[MAX_TABS];
	struct ether_header *ethh = (void *)frame->buf + 7;
	struct ip *iph = (void *) ethh + sizeof(struct ether_header);
	struct udphdr *udph = (void *) iph + iph->ip_hl*4;
	UDP_datagram *iperfh;
	int tabs = 0;
	
	memset(tab_pad, 0, MAX_TABS);
	
	print_eth_vector_decoded(frame, param);
	pad = &pad[strlen(pad)];
	
	while (tabs < 3)
		tab_pad[tabs++] = '\t';
	
	/* iperf UDP payload */
	iperfh = (void *) udph + sizeof(struct udphdr);
	sprintf(pad, "\n%sIperf:\n", tab_pad);
	pad = &pad[strlen(pad)];
	tab_pad[tabs++] = '\t';
	sprintf(pad, "%sID: %d\n%sTimestamp (sec, usec): %u %u\n", 
			tab_pad, ntohl(iperfh->id), tab_pad, ntohl(iperfh->tv_sec), 
			ntohl(iperfh->tv_usec));
	pad = &pad[strlen(pad)];
	
	/* prep the return value, print_eth_vector_decoded already set the ip_id */
	ids->iperf_id = ntohl(iperfh->id);
}

#endif /* _DECODECOMMON_H_ */
