
/*
 * Copyright (C) 2009 Tudor Marian <tudor.marian@cornell.edu>
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include "scramblercommon.h"
#include "common64b66b.h"
#include "decodecommon.h"

#define TTL	64

typedef void (*payload_fn)(int k, int size, void *packet, void *state);

void cycle_payload(int k, int size, void *packet, void *state)
{
	char cycle[] = "6789012345", *payload = packet;
	int i, len_cycle = strlen(cycle);

	for (i = 0; i < size; i++)
		payload[i] = cycle[i % len_cycle];
}

void iperf_payload(int k, int size, void *packet, void *state)
{
	UDP_datagram *hdr1 = packet;
	client_hdr *hdr2 = packet + sizeof(UDP_datagram);
	char *payload = (void *) hdr2 + sizeof(client_hdr);
	const char cycle[] = "6789012345";
	int i, len_cycle = strlen(cycle);
	
	size -= (sizeof(UDP_datagram)+sizeof(client_hdr));
	memset(hdr1, 0, sizeof(*hdr1));
	memset(hdr2, 0, sizeof(*hdr2));
	
	hdr1->id = htonl(k);
	hdr1->tv_sec = htonl(1247146893);
	hdr1->tv_usec = htonl(163394);
	
	for (i = 0; i < size; i++)
		payload[i] = cycle[i % len_cycle];
}

void udp_packet(int k, int size, void *packet, void *state, uint16_t sport, 
	uint16_t dport, struct ip *iph, payload_fn fn)
{
	int hdr_len = UDP_HDR_LEN;
	struct udphdr *udph = packet;
	
	fn(k, size-hdr_len, packet+hdr_len, state);
	
	udph->uh_sport = htons(sport);
	udph->uh_dport = htons(dport);
	udph->uh_ulen = htons(size);
	udph->uh_sum = 0;
	udph->uh_sum = htons(udp_csum(udph, iph));
}

void ipv4_packet(int k, int size, void *packet, void *state, uint16_t ipid, 
	char *saddr, char *daddr, uint16_t sport, uint16_t dport, payload_fn fn)
{
	int hdr_len = IP_HDR_LEN;
	struct ip *iph = packet;
	struct in_addr *paddr;
	
	iph->ip_hl = hdr_len / 4;
	iph->ip_v = 4;
	iph->ip_tos = 0;
	iph->ip_len = htons(size);
	iph->ip_id = htons(ipid);
	iph->ip_off = htons(2 << (5+8));
	iph->ip_ttl = TTL;
	iph->ip_p = IPPROTO_UDP;
	iph->ip_sum = 0;
	
	paddr = &iph->ip_src;
	inet_pton(AF_INET, saddr, paddr);
	paddr = &iph->ip_dst;
	inet_pton(AF_INET, daddr, paddr);
	
	/* the IP src and dst addresses must be filled in before this call */
	udp_packet(k, size-hdr_len, packet+hdr_len, state, sport, dport, iph, fn);
	
	iph->ip_sum = htons(csum(iph, iph->ip_hl*4));
}

void load_ether_addr(const char *mac, u_char *ether_addr)
{
	char *mac_str, *p, *sep;
	
	if (!(mac_str = malloc(6 * 2 + 5)))
	{
		fprintf(stderr, "malloc error %s\n", strerror(errno));
		exit(EXIT_FAILURE);
	}	
	strcpy(mac_str, mac);
	
	sep = mac_str;
	while ((p = strsep(&sep, ":")) != NULL)
		sscanf(p, "%hhx", ether_addr++);
		
	free(mac_str);
}

void eth_packet(int k, int size, void *packet, void *state, 
	char *mac_src, char *mac_dst, uint16_t ipid, char *saddr, char *daddr, 
	uint16_t sport, uint16_t dport, payload_fn fn)
{
	int hdr_len = ETH_PREAMBLE_HDR_LEN, i;
	struct ether_header *ethh = packet + 7;
	uint32_t *crc_bytes = packet + size - 4;
	
	ipv4_packet(k, size-ETH_PREAMBLE_HDR_CRC_LEN, packet+hdr_len, state, ipid, 
		saddr, daddr, sport, dport, fn);
	
	for (i = 0; i < 6; i++)
		((unsigned char *) packet)[i] = 0x55;
	((unsigned char *) packet)[i] = 0xd5;
	
	ethh->ether_type = htons(ETHER_TYPE_IPV4);
	load_ether_addr(mac_dst, ethh->ether_dhost);
	load_ether_addr(mac_src, ethh->ether_shost);
	
	*crc_bytes = crc32_le(INIT_CRC, (void *) ethh, 
		((unsigned long)crc_bytes) - ((unsigned long) ethh)) ^ INIT_CRC;
}

int main(int argc, char *argv[])
{
	int i;
	char *mac_src = NULL, *mac_dst = NULL, *saddr = NULL, *daddr = NULL, 
		packet[MAX_MTU], print_buf[MAX_MTU];
	uint16_t sport = 0, dport = 0, ip_id = 0;
	int eth_mtu = DEFAULT_ETH_MTU-2, iperf_id = -1, count = 0;
	struct frame_bytes fr;
	payload_fn fn = cycle_payload;
	
	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "--mac-src") == 0)
			mac_src = argv[++i];
		else if (strcmp(argv[i], "--mac-dst") == 0)
			mac_dst = argv[++i];
		else if (strcmp(argv[i], "--saddr") == 0)
			saddr = argv[++i];
		else if (strcmp(argv[i], "--daddr") == 0)
			daddr = argv[++i];
		else if (strcmp(argv[i], "--sport") == 0)
			sport = (uint16_t) atoi(argv[++i]);
		else if (strcmp(argv[i], "--dport") == 0)
			dport = (uint16_t) atoi(argv[++i]);
		else if (strcmp(argv[i], "--ip-id") == 0)
			ip_id = (uint16_t) atoi(argv[++i]);
		else if (strcmp(argv[i], "--eth-mtu") == 0)
			eth_mtu = atoi(argv[++i]);
		else if (strcmp(argv[i], "--iperf-id") == 0)
			iperf_id = atoi(argv[++i]);
		else if (strcmp(argv[i], "--count") == 0)
			count = atoi(argv[++i]);
		else
			goto out_usage;
	}
	
	if (eth_mtu < MIN_MTU || eth_mtu > MAX_MTU)
	{
		fprintf(stderr, "Invalid eth MTU size\n");
		goto out_usage;
	}
	
	if (!mac_src || !mac_dst || !saddr || !daddr || !sport || !dport)
	{
		out_usage:
		fprintf(stderr, "Usage <%s> [--mac-src mac] [--mac-dst mac] [--saddr ip] "\
			"[--daddr ip] [--sport port] [--dport port] [--ip-id id] "\
			"[--eth-mtu MTU] [--iperf-id id] [--count packet-count]\n", argv[0]);
		fprintf(stderr, "\te.g.: %s --mac-src 00:1b:21:1c:33:f8 "\
			"--mac-dst 00:1b:21:1c:34:06 --saddr 192.168.0.1 --daddr 192.168.0.2 "\
			"--sport 45378 --dport 5001 --ip-id 20835 --eth-mtu 1500 "\
			"--iperf-id 3456456 --count 1\n", argv[0]);
		exit(EXIT_FAILURE);
	}
	
	/* provision for the preamble, header and CRC */
	eth_mtu += ETH_PREAMBLE_HDR_CRC_LEN;
	fr.buf = packet;
	fr.len = eth_mtu;	

	if (iperf_id >= 0)
		fn = iperf_payload;
	
	for (i = 0; i < count; i++)
	{
		eth_packet(iperf_id+i, eth_mtu, packet, NULL, mac_src, mac_dst, ip_id+i, 
			saddr, daddr, sport, dport, fn);
		raw_eth_vector_decoded(&fr, print_buf);
		fprintf(stdout, "%s", print_buf);
	}
	
	exit(EXIT_FAILURE);
}
