
# Copyright (C) 2009 Tudor Marian <tudor.marian@cornell.edu>

import sys,struct,socket,random
from decodecommon import *
from decode64b66b import *

TTL = 64

iperf_cycle = '6789012345'
def cycle_payload(k, size, state):
	cycle_len = len(iperf_cycle)
	s = ''
	for i in range(size):
		s += iperf_cycle[i % cycle_len]
	return (s, state)
#end cycle_payload

def rand_payload(k, size, state):
	s = ''
	for i in range(size):
		s += chr(random.randint(0x0, 0xff))
	return (s, state)
#end rand_payload

def iperf_payload(k, size, state):
	secs = socket.ntohl(1247146893)
	usecs = socket.ntohl(163394)
	packet_id = socket.ntohl(k)
	# pack Iperf UDP_datagram
	payload = struct.pack('III', packet_id, secs, usecs)
	# pack Iperf client_hdr
	flags = numThreads = mPort = bufferlen = mWinBand = mAmount = 0
	payload += struct.pack('IIIIII', flags, numThreads, mPort, bufferlen, 
		mWinBand, mAmount)
	if state:
		assert isinstance(state, str)
		if len(state) > size-len(payload):
			del state[:size-len(payload)]
		elif len(state) < size-len(payload):
			bytes, nada = cycle_payload(k, size-len(payload)-len(state), state)
			state += bytes
		else:
			pass
	else:
		bytes, nada = cycle_payload(k, size-len(payload), state)
		state = bytes
	payload += state
	assert len(payload) == size
	return (payload, state)
#end iperf_payload

def udp_packet(k, size, state, sport, dport, ip_header_addrs, payload_fn):
	hdr_len = UDP_HDR_LEN
	payload, state = payload_fn(k, size-hdr_len, state)
	header = ''
	header += chr((sport >> 8) & 0xff) + chr(sport & 0xff)
	header += chr((dport >> 8) & 0xff) + chr(dport & 0xff)
	length = hdr_len+len(payload)
	header += chr((length >> 8) & 0xff) + chr(length & 0xff)
	check = udp_csum(header, length, ip_header_addrs, payload)
	header += chr((check >> 8) & 0xff) + chr(check & 0xff)
	#assert payload[12:] == iperf_s_test
	return (header+payload, state)
#end udp_packet

def ipv4_packet(k, ipid, size, state, saddr, daddr, sport, dport, payload_fn):
	hdr_len = IP_HDR_LEN
	header_addrs = socket.inet_aton(saddr) + socket.inet_aton(daddr)
	payload, state = udp_packet(k, size-hdr_len, state, sport, 
		dport, header_addrs, payload_fn)
	header = ''
	header += chr ( (4 << 4) | ((hdr_len / 4) & 0xf) ) # version & header length
	header += chr(0x00) # TOS
	length = hdr_len+len(payload)
	header += chr((length >> 8) & 0xff) + chr(length & 0xff)
	header += chr((ipid >> 8) & 0xff) + chr(ipid & 0xff)
	flags_and_offset = 2 << (5+8)
	header += chr((flags_and_offset >> 8) & 0xff) + chr(flags_and_offset & 0xff)
	header += chr(TTL)
	header += chr(IPPROTO_UDP)
	options = ''
	csum = ipv4_header_csum(header, header_addrs, options)
	header += chr((csum >> 8) & 0xff) + chr(csum & 0xff)
	return (header+header_addrs+options+payload, state)
#end ip_packet

def eth_packet(k, mac_src, mac_dst, size, ipid, state, saddr, daddr, sport, 
	dport, payload_fn):
	hdr_len = ETH_PREAMBLE_HDR_CRC_LEN
	payload, state = ipv4_packet(k, ipid, size-hdr_len, state, saddr, daddr, 
		sport, dport, payload_fn)
	frame_prefix = 6 * chr(0x55) + chr(0xd5)
	frame = ''
	for byte_str in mac_dst.split(':'):
		frame += chr(int(byte_str, 16))
	for byte_str in mac_src.split(':'):
		frame += chr(int(byte_str, 16))
	frame += chr((ETHER_TYPE_IPV4 >> 8) & 0xff) + chr(ETHER_TYPE_IPV4 & 0xff)
	frame += payload
	crc_le = crc32_le(INIT_CRC, frame) ^ INIT_CRC
	for i in range(4):
		# bytes reversed, since I used the crc32_le (little endian)
		frame += chr( (crc_le >> (8*i) ) & 0xff )
	return (frame_prefix + frame, state)
#end eth_packet

def genpackets(n, kbase, size, param):
	(mac_src, mac_dst, ipid_base, saddr, daddr, sport, dport, payload_fn) = param
	packets = []
	state = None
	for i in range(n):
		frame, state = eth_packet(kbase+i, mac_src, mac_dst, size, ipid_base+i, 
			state, saddr, daddr, sport, dport, payload_fn)
		packets.append(frame)
	return packets
#end genpackets

def out_usage():
	sys.stderr.write("Usage <%s> [--mac-src mac] [--mac-dst mac] [--saddr ip] "\
		"[--daddr ip] [--sport port] [--dport port] [--ip-id id] "\
		"[--eth-mtu MTU] [--iperf-id id] [--count packet-count]\n" % sys.argv[0]);
	sys.stderr.write("\te.g.: %s --mac-src 00:1b:21:1c:33:f8 "\
		"--mac-dst 00:1b:21:1c:34:06 --saddr 192.168.0.1 --daddr 192.168.0.2 "\
		"--sport 45378 --dport 5001 --ip-id 20835 --eth-mtu 1500 "\
		"--iperf-id 3456456 --count 1\n" % sys.argv[0]);
	sys.exit(1)
#end out_usage

def main():
	mac_src = mac_dst = saddr = daddr = sport = dport = None
	eth_mtu = DEFAULT_ETH_MTU-2
	ip_id = count = i = 1
	iperf_id = -1
	
	while i < len(sys.argv):
		if sys.argv[i] == "--mac-src":
			i += 1
			mac_src = sys.argv[i]
		elif sys.argv[i] == "--mac-dst":
			i += 1
			mac_dst = sys.argv[i]
		elif sys.argv[i] == "--saddr":
			i += 1
			saddr = sys.argv[i]
		elif sys.argv[i] == "--daddr":
			i += 1
			daddr = sys.argv[i]
		elif sys.argv[i] == "--sport":
			i += 1
			sport = int(sys.argv[i])
		elif sys.argv[i] == "--dport":
			i += 1
			dport = int(sys.argv[i])
		elif sys.argv[i] == "--ip-id":
			i += 1
			ip_id = int(sys.argv[i])
		elif sys.argv[i] == "--eth-mtu":
			i += 1
			eth_mtu = int(sys.argv[i])
		elif sys.argv[i] == "--iperf-id":
			i += 1
			iperf_id = int(sys.argv[i])
		elif sys.argv[i] == "--count":
			i += 1
			count = int(sys.argv[i])
		else:
			out_usage()
		i += 1
	
	if eth_mtu < MIN_MTU or eth_mtu > MAX_MTU:
		sys.stderr.write("Invalid eth MTU size\n")
		out_usage()
	
	if not mac_src or not mac_dst or not saddr or not daddr or \
		not sport or not dport:
		out_usage()
	
	payload_fn = iperf_payload if iperf_id >= 0 else cycle_payload
	
	eth_frame_size = ETH_PREAMBLE_HDR_CRC_LEN + eth_mtu
	param = (mac_src, mac_dst, ip_id, saddr, daddr, sport, dport, payload_fn)
	packets = genpackets(count, iperf_id, eth_frame_size, param)
	packet_list_to_file(packets, sys.stdout)
#end main

if __name__ == '__main__':
	main()
