
# Copyright (C) 2009 Tudor Marian <tudor.marian@cornell.edu>

from decode64b66b import *
from hbitstream import *

eth_test_vector = '\x08\x00\x20\x77\x05\x38\x0e\x8b\x00\x00\x00\x00\x08'+\
	'\x00\x45\x00\x00\x28\x1c\x66\x00\x00\x1b\x06\x9e\xd7\x00\x00\x59\x4d\x00'+\
	'\x00\x68\xd1\x39\x28\x4a\xeb\x00\x00\x30\x77\x00\x00\x7a\x0c\x50\x12'+\
	'\x1e\xd2\x62\x84\x00\x00\x00\x00\x00\x00\x00\x00\x93\xeb\xf7\x79'

def eth_vector_decoded(vector):
	for c in vector:
		sys.stderr.write('%02x ' % ord(c))
	sys.stderr.write('\n')

def eth_found_vector(vector):
	if vector == eth_test_vector:
		print 'Correct decoding!'
	else:
		print 'Incorrect decoding!'
		for c in vector:
			sys.stderr.write('%02x ' % ord(c))
		sys.stderr.write('\n')

def main():
	if len(sys.argv) != 3:
		sys.stdout.write('Usage %s <first-number> <second-number>\n' % \
			sys.argv[0])
		sys.exit()
	# Test scrambling & descrambling
	
	first_number = int(sys.argv[1])
	second_number = int(sys.argv[2])
	scrambler_state = random.randint(0, 0xffffffffffffffff)
	#descrambler_state = scrambler_state
	descrambler_state = random.randint(0, 0xffffffffffffffff)
	print('Init state for scrambler 0x%x' % (scrambler_state))
	print('Init state for de-scrambler 0x%x' % (descrambler_state))
	
	scrambler_state, scrambled_number = \
		scrambler(scrambler_state, first_number, scramble)
		
	descrambler_state, descrambled_number = \
		scrambler(descrambler_state, scrambled_number, descramble)
		
	if (first_number == descrambled_number):
		print 'First Scrambling OK'
	else:
		print 'First Scrambling FAILED!'
		
	# 2nd number, keep the state
	scrambler_state, scrambled_number = \
		scrambler(scrambler_state, second_number, scramble)
	
	descrambler_state, descrambled_number = \
		scrambler(descrambler_state, scrambled_number, descramble)
		
	if (second_number == descrambled_number):
		print 'Second Scrambling OK'
	else:
		print 'Second Scrambling FAILED!'
	
	# third number --- walker test vector
	print '\nWalker test vector...'
	scrambler_state = 0xffffffffffffffff
	third_number = 0x1e00000000000000
	scrambled_third = 0x1e00000080f0ff7b
	
	scrambler_state, scrambled_number = \
		scrambler(scrambler_state, third_number, scramble)
	
	if scrambled_number == scrambled_third:
		print 'Proper de-scrambling!'
	else:
		print 'Incorrect de-scrambling!'
		sys.exit(1)
	# Walker
	# test vector (scrambled)
	scrambled_test_vector = [\
		(0x2, 0x1e00000080f0ff7b), (0x2, 0x7848f8df88c6f313), \
		(0x1, 0xc5a59cda93251d1f), (0x1, 0x7abbd0ddffc9211b), \
		(0x1, 0x0523446faa5c30b6), (0x1, 0xbb2d4288815625cf), \
		(0x1, 0x7278fd943dd68ce5), (0x1, 0x39f5ea522eafec92), \
		(0x1, 0xfd3c1ccf755fbe8f), (0x2, 0x8db31308ba3fe73f)]
	
	#sys.setrecursionlimit(16384)
	print '\nDecoding over example vector...'
	try:
		decode(scrambled_test_vector, eth_found_vector)
		decode(hbitstream, eth_vector_decoded)
	except DecodeException, e:
		sys.stderr.write(e)
# end main

if __name__ == "__main__":
	main()
